import os

import pytest
from dotenv import load_dotenv
from mm_web3 import fetch_proxies_sync
from typer.testing import CliRunner

load_dotenv()

PROXIES_URL = os.getenv("PROXIES_URL")


@pytest.fixture()
def mnemonic() -> str:
    return "hub blur cliff taste afraid master game milk nest change blame code"


@pytest.fixture
def passphrase() -> str:
    return "my-secret"


@pytest.fixture()
def mainnet_bip44_address_0() -> str:
    return "1Men3kiujJH7H5NXyKpFtWWtni1cTfSk48"


@pytest.fixture
def binance_address() -> str:
    return "34xp4vRoCGJym3xR7yCVPFHoCNxv4Twseo"


@pytest.fixture
def proxies() -> list[str]:
    return fetch_proxies_sync(PROXIES_URL).unwrap("cannot fetch proxies")


@pytest.fixture
def cli_runner() -> CliRunner:
    return CliRunner()



from bitcoinlib.transactions import Transaction


def decode_tx(tx_hex: str, testnet: bool = False) -> dict[str, object]:
    return Transaction.parse(tx_hex, network="testnet" if testnet else "mainnet").as_dict()  # type: ignore[no-any-return]



from pathlib import Path

import mm_print
from bit import PrivateKey, PrivateKeyTestnet
from mm_web3 import Web3CliConfig

from mm_btc.wallet import is_testnet_address


class Config(Web3CliConfig):
    class Output(Web3CliConfig):
        address: str
        amount: int

    from_address: str
    private: str
    outputs: list[Output]


def run(config_path: Path) -> None:
    config = Config.read_toml_config_or_exit(config_path)
    testnet = is_testnet_address(config.from_address)
    key = PrivateKeyTestnet(config.private) if testnet else PrivateKey(config.private)

    outputs = [(o.address, o.amount, "satoshi") for o in config.outputs]

    tx = key.create_transaction(outputs)
    mm_print.json(tx)



from mm_btc.wallet import (
    BIP44_MAINNET_PATH,
    BIP44_TESTNET_PATH,
    BIP84_MAINNET_PATH,
    BIP84_TESTNET_PATH,
    AddressType,
    derive_accounts,
    generate_mnemonic,
)


def test_generate_mnemonic():
    mnemonic = generate_mnemonic(words=24)
    assert len(mnemonic.split()) == 24

    assert generate_mnemonic() != generate_mnemonic()


def test_derive_accounts(mnemonic, passphrase):
    # mainnet bip44
    accs = derive_accounts(mnemonic, passphrase, BIP44_MAINNET_PATH, AddressType.P2PKH, 2)
    assert accs[1].path == "m/44'/0'/0'/0/1"
    assert accs[1].address == "13MJrGQq8RwLcGfdnY58666kCsZqevsadE"
    assert accs[1].private == "0d5feb50af849f2ebcc0c83a4446ddbceefd7b0ceba437a57161fa37702d096e"
    assert accs[1].wif == "Kwfi75WxsuaKnmxRUQDoEZDzKXkMC8E9Pzk3vVjEdPsJu1DXCFPv"

    # mainnet bip84
    accs = derive_accounts(mnemonic, passphrase, BIP84_MAINNET_PATH, AddressType.P2WPKH, 2)
    assert accs[1].path == "m/84'/0'/0'/0/1"
    assert accs[1].address == "bc1qszqeg6gfcmt8zurkeayes2895urnea2m54rt3w"
    assert accs[1].private == "1cc57553f9e3e1b863693be92ac753f682f61f1a2dede2ceff97262027635334"
    assert accs[1].wif == "KxBdzQTQduYKoec1FWgwMDF9R1a4UdQp3ezyhjniw4W2k54Cb6gE"

    # testnet bip44
    accs = derive_accounts(mnemonic, passphrase, BIP44_TESTNET_PATH, AddressType.P2PKH, 2)
    assert accs[1].path == "m/44'/1'/0'/0/1"
    assert accs[1].address == "miRfXh2aH6pjhdo5EoBt4RRdAipR88dcV3"
    assert accs[1].private == "3a2e8fc45c75ab8a7cdafacf7c77c941f8eebc813ee2f0e17ee86e7ac8584b3a"
    assert accs[1].wif == "cPXoLA4cRKxeht4iH6ZsCDQxPspFi8SEieZNTKDzGgog94Zg5Y64"

    # testnet bip84
    accs = derive_accounts(mnemonic, passphrase, BIP84_TESTNET_PATH, AddressType.P2WPKH, 2)
    assert accs[1].path == "m/84'/1'/0'/0/1"
    assert accs[1].address == "tb1qp36pk9w408pgfpwn5myxwurr3fhzryn0lpyah2"
    assert accs[1].private == "150fd20e4cb377ae57b43fed59ca2765a9f5d91a20640a4cfdf43ceb5403d48c"
    assert accs[1].wif == "cNHeFPcfq2XFfDAgq9icSK8NX29C1exPHekqYQdWgruQ6gPMBmXD"



import pytest

from mm_btc import blockstream
from mm_btc.blockstream import BlockstreamClient

pytestmark = pytest.mark.asyncio


async def test_get_address(binance_address, proxies):
    client = BlockstreamClient(proxies=proxies, timeout=5, attempts=5)

    # non-empty address
    res = await client.get_address(binance_address)
    assert res.unwrap().chain_stats.tx_count > 800

    # empty address
    res = await client.get_address("bc1pa48c294qk7yd7sc8y0wxydc3a2frv5j83e65rvm48v3ej098s5zs8kvh6d")
    assert res.unwrap().chain_stats.tx_count == 0

    # invalid address
    res = await client.get_address("bc1pa48c294qk7yd7sc8y0wxydc3a2frv5j83e65rvm48v3ej098s5zs8kvh5d")
    assert res.unwrap_err() == blockstream.ERROR_400_BAD_REQUEST

    # invalid network
    res = await client.get_address("mqkwWDWdgXhYunfoKvQfYyydwB5vdma3cK")
    assert res.unwrap_err() == blockstream.ERROR_400_BAD_REQUEST


async def test_get_confirmed_balance(binance_address, proxies):
    client = BlockstreamClient(proxies=proxies)
    res = await client.get_confirmed_balance(binance_address)
    assert res.unwrap() > 0
